@@ -14,7 +14,7 @@ for more details.
 You should have received a copy of the GNU General Public License
 along with notmuch.  If not, see <http://www.gnu.org/licenses/>.
 
-Copyright 2012-2014
+Copyright 2012-2017
 Jameson Rollins <jrollins@finestructure.net>
 """
 
@@ -23,10 +23,10 @@ import sys
 import codecs
 import signal
 
-import cli
-from source import Sources, SourceError
-from bibtex import Bibtex, BibtexError
-from parser import ParseError
+from . import cli
+from .source import Sources, SourceError
+from .bibtex import Bibtex, BibtexError
+from .parser import ParseError
 
 ########################################################################
 
@@ -60,6 +60,7 @@ Commands:
     --output=[summary|bibtex|tags|sources|keys|files]
                                         output format (default is 'summary')
     --limit=N                           limit number of results returned
+  tags <search-terms>                 Short for \"search --output=tags\".
   bibtex <search-terms>               Short for \"search --output=bibtex\".
   view <search-terms>                 View search in curses UI.
   count <search-terms>                Count matches.
@@ -294,6 +295,14 @@ if __name__ == '__main__':
         with cli.initdb() as db:
             cli.search(db, query, oformat=oformat, limit=limit)
 
+    ########################################
+    elif cmd in ['tags']:
+        argc = 2
+        query = make_query_string(sys.argv[argc:], require=False)
+        set_stdout_codec()
+        with cli.initdb() as db:
+            cli.search(db, query, oformat='tags')
+
     ########################################
     elif cmd in ['bibtex','bib','b']:
         argc = 2
